''*****************************************************************************
''
''	Platformer tutorial 101
''
''	Chapter 03-D
''  Delta Time using the Accumulator
''
''	Relminator (Richard Eric M. Lope)
''	http://rel.phatcode.net
''
''
''  I'm using EASY GL2D for rendering.
'' 	So try to read these tutorials for Easy GL2D..
'' 	http://back2basic.phatcode.net/?Issue_%232:Basic_2D_Rendering_in_OpenGL_using_Easy_GL2D%3A_Part_1
'' 	http://back2basic.phatcode.net/?Issue_%232:Basic_2D_Rendering_in_OpenGL_using_Easy_GL2D%3A_Part_2

''*****************************************************************************
#include once "fbgfx.bi"
#include once "FBGL2d7.bi" 
#include once "FBGL2d7.bas" 

const as integer SCREEN_WIDTH = 640
const as integer SCREEN_HEIGHT = 480


const as integer FALSE = 0
const as integer TRUE = not FALSE
const as single PI = atn(1) * 4

Type Ball
	
	declare constructor()
	declare sub Update( byval dt as single )
	declare sub Draw( byval Colour as integer )
	
	as single X
	as single Y
	as single Speed
	as single Dx
	as single Dy
	
End Type

constructor Ball()

	x = rnd * SCREEN_WIDTH
	y = rnd * SCREEN_HEIGHT
	dim as single a = (rnd * 360) * PI / 180  
	Speed = 80 + rnd * 180
	Dx = cos(a) * Speed
	Dy = sin(a) * Speed

End Constructor

sub Ball.Update( byval dt as single )
	
	X += Dx * dt		'' scale our movement by how much time
	Y += Dy * dt		'' has elapsed since last update
	
	'' bounce around if we hit a wall
	if( X < 0 ) then Dx = -Dx
	if( X > SCREEN_WIDTH ) then Dx = -Dx
	if( Y < 0 ) then Dy = -Dy
	if( Y > SCREEN_HEIGHT ) then Dy = -Dy
	  
End Sub

sub Ball.Draw( byval Colour as integer )
	
	GL2D.Circle( x, y, 20, Colour )

End Sub


''*****************************************************************************
'' Our main sub
''*****************************************************************************
sub main()

	
	GL2D.ScreenInit( SCREEN_WIDTH, SCREEN_HEIGHT )   ''Set up GL screen
	
	const as double FIXED_TIME_STEP = 1/60.0
	
	
	dim as Ball Balls( 30 )
	
	'' Dt stuff variables
	dim as single dt = 0
	dim as single Accumulator = 0
	dim as single LastTime = Timer
	dim as single CurrentTime = Timer
	dim as single SecondsElapsed = 0
	dim as single StartTime = Timer
	
	dim as integer Frame = 0        '' Display Frames
	dim as single FPS = 0			'' Display FPS
	
	dim as integer LogicFrames = 0	'' Physics Frames
	dim as single LogicFPS = 0		'' Physics FPS
	
	do
		
		'' Calculate deltatime
		CurrentTime = Timer
		dt = CurrentTime - LastTime
		LastTime = CurrentTime
		if( dt > FIXED_TIME_STEP ) then dt = FIXED_TIME_STEP    '' Limit dt
		SecondsElapsed += dt
		Accumulator += dt		'' Add accumulator
		
		Frame += 1				'' render frames
		
		'' As long as accumulator gets above 1/60 we update by dt = 1/60
		while( Accumulator >= FIXED_TIME_STEP )
			for i as integer = 0 to ubound(balls)
				Balls(i).Update( FIXED_TIME_STEP )   '' Update ball at fixed timesetep (1/60)
			next
			Accumulator -= FIXED_TIME_STEP		'' decrease accumulator
			LogicFrames += 1					'' Increase physics frame
		Wend
		
		GL2D.ClearScreen()
			
		GL2D.Begin2D()
	
			for i as integer = 0 to ubound(balls)
				Balls(i).Draw( GL2D_RGBA(255,255,255,255) )
			Next
			
			'' Calculate FPS
			dim as single Elapsed = CurrentTime - StartTime 
			if( Elapsed > 1 ) then
				FPS = Frame/Elapsed
				LogicFPS = LogicFrames/Elapsed
				Frame = 0
				LogicFrames = 0
				StartTime = timer
			EndIf
			GL2D.PrintScale(0, 0, 1, "Display FPS = " & FPS)
			GL2D.PrintScale(0, 10, 1, "Logic FPS = " & LogicFPS )
			GL2D.PrintScale(0, 30, 1, "Delta timing with Accumulator")
			
		GL2D.End2D()
		
		'sleep 1,1
		flip
		
	Loop until multikey(FB.SC_ESCAPE)


	GL2D.ShutDown()
	
End Sub

''*****************************************************************************
''*****************************************************************************


main()


end


